@@ -210,17 +210,18 @@ dates_zoom_in_cb (GtkButton *button, DatesData *data)
 		case 5:
 		case 7:
 		case 8:
-		case 9:
-		case 10:
 			data->zoom = 6;
 			break;
-
+		case 9:
+		case 10:
+			data->zoom = 7;
+			break;
 		case 11:
 		case 12:
 		case 13:
 		case 14:
 		case 15:
-			data->zoom = 8;
+			data->zoom = 10;
 			break;
 
 		case 16:
@@ -256,17 +257,17 @@ dates_zoom_out_cb (GtkButton *button, DatesData *data)
 			break;
 
 		case 6:
-			data->zoom = 8;
+			data->zoom = 7;
 			break;
-			
 		case 5:
 		case 7:
 		case 8:
-			data->zoom = 11;
+			data->zoom = 10;
 			break;
-
 		case 9:
 		case 10:
+			data->zoom = 11;
+			break;
 		case 11:
 		case 12:
 		case 13:
@@ -312,75 +313,72 @@ dates_date_changed_cb (DatesView *view, DatesData *d)
     gtk_label_set_markup (GTK_LABEL (d->header_label), buffer);
 }
 
-
 void
 dates_about_cb (GtkWidget *widget, DatesData *d)
 {
-    GdkPixbuf *pixb;
-    GError *error = NULL;
+	static GdkPixbuf *pixb;
+	GError *error = NULL;
 
-    /* NOTE: The hildon version of the about menu does not handle the
-     * credits button well (it is misplaced and not thematised), so we add
-     * the authors to the (c) string.
-     */
+	/* NOTE: The hildon version of the about menu does not handle the
+	 * credits button well (it is misplaced and not thematised), so we add
+	 * the authors to the (c) string.
+	 */
 #ifndef WITH_HILDON
-    const gchar *authors[] = {
-	"Principal author: Chris Lord <chris@o-hand.com>",
-	"Maemo port: Tomas Frydrych <tf@o-hand.com>",
-	NULL
-    };
+	const gchar *authors[] = {
+		"Principal author: Chris Lord <chris@o-hand.com>",
+		"Maemo port: Tomas Frydrych <tf@o-hand.com>",
+		"Rob Bradford <rob@o-hand.com>",
+		NULL
+	};
 
-    const gchar *licence =
-	"This program is free software; you can redistribute it and/or modify\n"
-	"it under the terms of the GNU General Public License as published by\n"
-	"the Free Software Foundation; either version 2, or (at your option)\n"
-	"any later version.\n"
-	"\n"
-	"This program is distributed in the hope that it will be useful,\n"
-	"but WITHOUT ANY WARRANTY; without even the implied warranty of\n"
-	"MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n"
-	"GNU General Public License for more details.";
+	const gchar *licence =
+		"This program is free software; you can redistribute it and/or modify\n"
+		"it under the terms of the GNU General Public License as published by\n"
+		"the Free Software Foundation; either version 2, or (at your option)\n"
+		"any later version.\n"
+		"\n"
+		"This program is distributed in the hope that it will be useful,\n"
+		"but WITHOUT ANY WARRANTY; without even the implied warranty of\n"
+		"MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n"
+		"GNU General Public License for more details.";
 #endif
+	
+	if (!pixb)
+		pixb = gdk_pixbuf_new_from_file (
+			PKGDATADIR G_DIR_SEPARATOR_S "oh-about-logo.png", &error);  
 
-    pixb = gdk_pixbuf_new_from_file (
-	PKGDATADIR G_DIR_SEPARATOR_S "oh-about-logo.png", &error);  
+	if (!pixb) {
+		g_assert (error);
+		g_warning (error->message);
+		g_clear_error (&error);
+	}
 
-    if (!pixb) {
-	g_assert (error);
-	g_warning (error->message);
-    }
+	gtk_show_about_dialog (
+			GTK_WINDOW (d->main_window),
+
+			"copyright",
+			"(c) 2006 OpenedHand Ltd",
 
-    gtk_show_about_dialog (
-	GTK_WINDOW (d->main_window),
 #ifndef WITH_HILDON
-	"copyright",
-	"(c) 2006 OpenedHand Ltd",
-
-	"authors",
-	authors,
+			"authors",
+			authors,
 			
-	"license",
-	licence,
-#else
-	"copyright",
-	"(c) 2006 OpenedHand Ltd\n"
-	"Authors: Chris Lord <chris@o-hand.com>,\n"
-	"Tomas Frydrych <tf@o-hand.com>\n",
+			"license",
+			licence,
 #endif
 
-	"logo",
-	pixb,
+			"logo",
+			pixb,
 
-	"name",
-	PACKAGE_NAME,
-
-	"version",
-	PACKAGE_VERSION,
-
-	"website",
-	"http://projects.o-hand.com/dates",
-
-	NULL);
+			"name",
+			PACKAGE_NAME,
+			
+			"version",
+			PACKAGE_VERSION,
+			
+			"website",
+			"http://www.pimlico-project.org/dates",
+			NULL);
 }
 
 
@@ -517,8 +515,8 @@ dates_event_selected_cb (DatesView *view, DatesData *d)
 	    g_warning ("Error accessing calendar list");
 	}
 
-	if (d->TBOpen)
-		gtk_widget_set_sensitive (d->TBOpen, !read_only);
+	if (d->TBEdit)
+		gtk_widget_set_sensitive (d->TBEdit, !read_only);
 
 	if (d->TBDelete)
 		gtk_widget_set_sensitive (d->TBDelete, !read_only);
@@ -528,8 +526,8 @@ dates_event_selected_cb (DatesView *view, DatesData *d)
 	if (!d->waiting)
 	    dates_platform_details_dlg (d, FALSE);
 
-	if (d->TBOpen)
-		gtk_widget_set_sensitive (d->TBOpen, FALSE);
+	if (d->TBEdit)
+		gtk_widget_set_sensitive (d->TBEdit, FALSE);
 
 	if (d->TBDelete)
 		gtk_widget_set_sensitive (d->TBDelete, FALSE);
@@ -919,7 +917,7 @@ dates_details_time_end_cb (GtkWidget *source, DatesData *d)
 }
 
 void
-dates_open_cb (GtkWidget *source, DatesData *d)
+dates_edit_cb (GtkWidget *source, DatesData *d)
 {
     if (d->read_only) return;
     /* TODO: Handle recurring events */
@@ -1238,9 +1236,9 @@ is_in_uids (GSList *uids, ESource *source)
 {
 	GSList *l;
 
-	for (l = uids; l; l = l->next) {
+	for (l = uids; l; l = l->next)
+	{
 		const char *uid = l->data;
-
 		if (strcmp (uid, e_source_peek_uid (source)) == 0)
 			return TRUE;
 	}
@@ -1255,6 +1253,7 @@ dates_cal_open_cb (ECal *ecal, ECalendarStatus status, DatesData *d)
     GError *error = NULL;
     const char *ecal_name;
     ESource *source;
+    ESourceGroup *group;
     GtkTreeIter iter;
     gchar *primary_uid;
     GSList *selected_list;
@@ -1286,72 +1285,6 @@ dates_cal_open_cb (ECal *ecal, ECalendarStatus status, DatesData *d)
 	
     source = e_cal_get_source (ecal);
 
-    /* Create the system calendar when it isn't found */
-    if (d->first_load)
-	{
-		gchar *group_uri;
-		ESourceGroup *group;
-		ESourceList *cal_list;
-		const gchar *cal_uri = e_cal_get_uri (ecal);
-		gint size;
-		GError *error = NULL;
-		
-#ifdef DEBUG
-		if (d->debug & DATES_DEBUG_CALENDAR)
-		{
-			g_debug ("First time run, "
-					 "creating new system calendar");
-			g_debug ("New system calendar URI: %s", cal_uri);
-		}
-#endif
-		
-		/* FIXME: This whole section is a little hacky, do it better */
-		
-		/* Create group by chopping off last bit of system calendar uri
-		 */
-		size = g_strrstr (cal_uri, "/system") - cal_uri;
-		group_uri = g_new (gchar, size);
-		strncpy (group_uri, cal_uri, size + 1);
-		group_uri[size] = '\0';
-#ifdef DEBUG
-		if (d->debug & DATES_DEBUG_CALENDAR)
-			g_debug ("Creating new group at %s", group_uri);
-#endif
-		group = e_source_group_new ("On This Computer", group_uri);
-		g_free (group_uri);
-		
-		/* Name system calendar and add it to group */
-		e_source_set_absolute_uri (source, cal_uri);
-		/* NOTE: Default Evolution colour */
-		e_source_set_color (source, 0xBECEDD);
-		e_source_set_name (source, "Personal");
-		e_source_group_add_source (group, source, 0);
-		
-		/* Add group to default list */
-		cal_list = e_source_list_new_for_gconf_default (
-			CALENDAR_GCONF_SOURCES);
-		e_source_list_add_group (cal_list, group, 0);
-
-		if (!e_source_list_sync (cal_list, &error))
-		{
-			g_warning ("Error syncing ESourceList: %s",
-					   error->message);
-			g_error_free (error);
-			return;
-		}
-		
-		g_signal_connect (G_OBJECT (cal_list), "changed",
-						  G_CALLBACK (dates_sources_changed_cb), d);
-		d->first_load = FALSE;
-		
-		/* Return, the sources will change immediately after this */
-#ifdef DEBUG
-		if (d->debug & DATES_DEBUG_CALENDAR)
-			g_debug ("Created new system calendar and group");
-#endif
-		return;
-    }
-
     ecal_name = e_source_peek_name (source);
 
     if (!e_cal_is_read_only (ecal, &read_only, &error))
@@ -1362,9 +1295,25 @@ dates_cal_open_cb (ECal *ecal, ECalendarStatus status, DatesData *d)
 		g_object_unref (ecal);
 		return;
     }
+
+	/* 
+	 * Work around the fact that eds-dbus seems to be returning the wrong
+	 * value for e_cal_is_read_only on the calendar itself for the http
+	 * backend. But on the events themselves it is fine
+	 */
+	group = e_source_peek_group (source);
+	if (strncmp (e_source_group_peek_base_uri (group), "webcal://", 9) == 0)
+		read_only = TRUE;
 	
     selected_list = gconf_client_get_list (gconf_client_get_default (),
 					   CALENDAR_GCONF_SELECTED, GCONF_VALUE_STRING, NULL);
+#ifdef DEBUG
+		if (d->debug & DATES_DEBUG_CALENDAR)
+			g_debug ("selected_list %p: is in uids %d",
+					 selected_list,
+					 is_in_uids (selected_list, source));
+#endif
+	
     if (selected_list && is_in_uids (selected_list, source))
 	{
 		dates_view_add_calendar (d->view, ecal);
@@ -1482,7 +1431,7 @@ dates_load_esource (ESource        *esource,
     }
 
     error = NULL;
-    if (!e_cal_open (retval, TRUE, &error))
+    if (!e_cal_open (retval, FALSE, &error))
     {
 	g_assert (error != NULL);
 	g_warning ("Cannot open calendar from uri '%s': %s\n",
@@ -1496,6 +1445,14 @@ dates_load_esource (ESource        *esource,
 	g_debug ("Loaded calendar from uri '%s'",
 		 e_cal_get_uri (retval));
 #endif
+
+    /* 
+     * Set the calendar mode to CAL_MODE_REMOTE. This is necessary to get the
+     * remote (webcal) calendars to work. It doesn't seem to have any effect
+     * on the file backend.
+     */
+    e_cal_set_mode (retval, CAL_MODE_REMOTE);
+
     return retval;
 }
 
@@ -1516,6 +1473,12 @@ dates_sources_changed_cb (ESourceList *cal_list, DatesData *d)
     if (d->debug & DATES_DEBUG_CALENDAR)
 	g_debug ("Sources changed, reloading calendars");
 #endif
+	if (d->first_load)
+	{
+		d->first_load = FALSE;
+		dates_autoselect_calendars (d, cal_list);
+	}
+
     dates_update_calendars (cal_list, d);
 }
 
@@ -1583,7 +1546,14 @@ dates_update_calendars (ESourceList *cal_list, DatesData *d)
 	    g_signal_connect (G_OBJECT (ecal), "cal_opened",
 			      G_CALLBACK (dates_cal_open_cb), d);
 
-	    e_cal_open_async (ecal, TRUE);
+	    e_cal_open_async (ecal, FALSE);
+	    
+	    /* 
+	     * Set the calendar mode to CAL_MODE_REMOTE. This is necessary to get the
+	     * remote (webcal) calendars to work. It doesn't seem to have any effect
+	     * on the file backend.
+	     */
+	    e_cal_set_mode (ecal, CAL_MODE_REMOTE);
 	}
     }	
 }
@@ -1592,12 +1562,9 @@ void
 dates_gconf_selected_cb (GConfClient *client, guint cnxn_id, GConfEntry *entry,
 			 gpointer user_data)
 {
-    ESourceList *cal_list;
     DatesData *d = (DatesData *)user_data;
 	
-    cal_list = e_source_list_new_for_gconf_default (CALENDAR_GCONF_SOURCES);
-	
-    dates_update_calendars (cal_list, d);
+    dates_update_calendars (d->source_list, d);
 }
 
 void
@@ -1693,3 +1660,163 @@ dates_button_press_cb (GtkWidget	*widget,
 	
 	return TRUE;
 }
+
+void
+dates_autoselect_calendars (DatesData *d, ESourceList * cal_list)
+{
+	GSList *selected_list;
+
+	if (!cal_list)
+		return;
+
+	selected_list = gconf_client_get_list (gconf_client_get_default (),
+					       CALENDAR_GCONF_SELECTED,
+					       GCONF_VALUE_STRING, NULL);
+
+	if (!selected_list) {
+		/* select all available calendars */
+		GSList *list = NULL, *groups, *g;
+		groups = e_source_list_peek_groups (cal_list);
+#ifdef DEBUG
+		if (d->debug & DATES_DEBUG_CALENDAR)
+			g_debug ("Attempting to autoselect");
+#endif
+		
+		for (g = groups; g; g = g->next) {
+			GSList *sources, *s;
+			sources =
+				e_source_group_peek_sources (E_SOURCE_GROUP (g->data));
+			for (s = sources; s; s = s->next) {
+				ESource *source = E_SOURCE (s->data);
+				ECal *ecal;
+				gchar * uid;
+			
+				ecal = dates_load_esource (source,
+							   E_CAL_SOURCE_TYPE_EVENT,
+							   NULL, d);
+				if (!ecal)
+				{
+#ifdef DEBUG
+					if (d->debug & DATES_DEBUG_CALENDAR)
+						g_debug ("No ecal");
+#endif
+					
+					continue;
+				}
+				
+				uid = (gchar *)e_source_peek_uid (e_cal_get_source (ecal));
+				list = g_slist_prepend (list, uid);
+			}
+		}
+
+		if (list)
+		{
+#ifdef DEBUG
+			if (d->debug & DATES_DEBUG_CALENDAR)
+				g_debug ("Setting new list");
+#endif
+			gconf_client_set_list (gconf_client_get_default (),
+					       CALENDAR_GCONF_SELECTED,
+					       GCONF_VALUE_STRING,
+					       list, NULL);
+		}
+	}
+	else
+#ifdef DEBUG
+		if (d->debug & DATES_DEBUG_CALENDAR)
+			g_debug ("Have selected list");
+#endif
+}
+
+gboolean
+dates_import_calendar_data_from_file (ECal *cal, gchar *filename, GError **error)
+{
+  GError *tmp_error = NULL;
+  GIOChannel *channel = NULL;
+  gchar *line;
+  gsize length;
+
+  channel = g_io_channel_new_file (filename, "r", &tmp_error);
+
+  if (tmp_error != NULL)
+  {
+    g_warning ("Error when opening file: %s", tmp_error->message);
+    g_propagate_error (error, tmp_error);
+    return FALSE;
+  } else {
+    GIOStatus status;
+    icalparser *parser = icalparser_new ();
+
+    /* Read the from the file line by line and until EOF */
+    while ((status = g_io_channel_read_line (channel, &line, &length, NULL, &tmp_error))
+        == G_IO_STATUS_NORMAL)
+    {
+      icalcomponent *icomp = NULL;
+
+      /* The parser returns an icalcomponent when it has one */
+      icomp = icalparser_add_line (parser, line);
+      g_free (line);
+
+    if (icomp)
+      {
+        gchar *uid = NULL;
+        icalcompiter iter;
+        icalcomponent *subcomp;
+
+        /* The component is a top-level one and e_cal_create_object only
+         * accepts VEVENTs. Iterate through the VEVENTS. */
+        iter = icalcomponent_begin_component (icomp, ICAL_VEVENT_COMPONENT);
+
+        while ((subcomp = icalcompiter_deref (&iter)) != NULL) 
+        {
+          if (!e_cal_create_object (cal, subcomp, &uid, &tmp_error))
+          {
+            g_warning ("Creation of imported event failed: %s", tmp_error->message);
+            g_propagate_error (error, tmp_error);
+
+            if (parser)
+              icalparser_free (parser);
+
+            if (icomp)
+              icalcomponent_free (icomp);
+
+            if (channel)
+              g_io_channel_unref (channel);
+
+            g_free (uid);
+            return FALSE;
+          }
+
+          icalcompiter_next (&iter);
+          g_free (uid);
+        }
+
+        icalcomponent_free (icomp);
+      }
+    }
+
+    if (parser)
+      icalparser_free (parser);
+
+    if (tmp_error != NULL)
+    {
+      g_warning ("Error when reading from file: %s", tmp_error->message);
+      g_propagate_error (error, tmp_error);
+
+      g_io_channel_unref (channel);
+      return FALSE;
+    }
+  }
+
+  if (channel)
+    g_io_channel_unref (channel);
+
+  return TRUE;
+}
+
+void
+dates_import_cb (GtkWidget *menu_item, gpointer user_data)
+{
+  dates_platform_import_dialog ((DatesData *)user_data);
+}
+
